@@ -482,7 +482,7 @@ class Beam(object):
             x = self.get_column(col, nolost=nolost)
             w = numpy.ones(len(x))
         else:
-            x, w = self.getshcol((col, ref), nolost=nolost)
+            x, w = self.get_columns((col, ref), nolost=nolost)
 
         if factor != 1.0: x *= factor
 
@@ -869,7 +869,7 @@ class Beam(object):
 
         """
 
-        print(">>>>translation: ", qdist1)
+        # print(">>>>translation: ", qdist1)
         if numpy.array(qdist1).size != 3:
             raise Exception("Input must be a vector [x,y,z]")
 
@@ -929,7 +929,7 @@ class Beam(object):
             self.rays[:,newtoroti[1]] = -a1[:,newtoroti[0]] * sinth + a1[:,newtoroti[1]] * costh
             self.rays[:,newaxisi]     =  a1[:,newaxisi]
 
-    def change_to_image_reference_system(self, theta, T_IMAGE, rad=True):
+    def change_to_image_reference_system(self, theta, T_IMAGE, rad=True, refraction_index=1.0):
         """
         Implements the propagation from the mirror reference frame to the screen (image) reference.
         Mimics IMREF and IMAGE1 subrutines in shadow3
@@ -954,7 +954,7 @@ class Beam(object):
 
         a1 = self.rays.copy()
 
-        print(">>>>> rotate_imref T_REFLECTION = ", T_REFLECTION, T_REFLECTION * 180 / numpy.pi)
+        # print(">>>>> rotate_imref T_REFLECTION = ", T_REFLECTION, T_REFLECTION * 180 / numpy.pi)
 
         nrays = self.rays.shape[0]
         UXIM_x = numpy.ones(nrays)
@@ -1014,7 +1014,7 @@ class Beam(object):
             self.rays[:, i - 1 + 2] = a1[:, i - 1 + 0] * VZIM_x   + a1[:, i - 1 + 1] * VZIM_y   + a1[:, i - 1 + 2] * VZIM_z
 
         # optical path col 13
-        self.rays[:, 12] += numpy.abs(DIST)
+        self.rays[:, 12] += numpy.abs(DIST) * refraction_index
     #
     # crop
     #
@@ -1252,6 +1252,9 @@ class Beam(object):
         -------
 
         """
+        if numpy.iscomplexobj(Rs):
+            print(">>> Warning: using complex reflectivities. Use apply_complex_reflectivity_s() instead")
+
         self.rays[:, 6] *= Rs
         self.rays[:, 7] *= Rs
         self.rays[:, 8] *= Rs
@@ -1267,6 +1270,10 @@ class Beam(object):
         -------
 
         """
+
+        if numpy.iscomplexobj(Rp):
+            print(">>> Warning: using complex reflectivities. Use apply_complex_reflectivity_p() instead")
+
         self.rays[:, 15] *= Rp
         self.rays[:, 16] *= Rp
         self.rays[:, 17] *= Rp
@@ -1286,7 +1293,65 @@ class Beam(object):
         self.apply_reflectivity_s(Rs)
         self.apply_reflectivity_p(Rp)
 
+    # complex reflectivities...
 
+    def apply_complex_reflectivity_s(self, Rs):
+        """
+
+        Parameters
+        ----------
+        Rs
+
+        Returns
+        -------
+
+        """
+        self.rays[:, 6] *= numpy.abs(Rs)
+        self.rays[:, 7] *= numpy.abs(Rs)
+        self.rays[:, 8] *= numpy.abs(Rs)
+        self.rays[:, 13] += numpy.angle(Rs)
+
+    def apply_complex_reflectivity_p(self, Rp):
+        """
+
+        Parameters
+        ----------
+        Rp
+
+        Returns
+        -------
+
+        """
+        self.rays[:, 15] *= numpy.abs(Rp)
+        self.rays[:, 16] *= numpy.abs(Rp)
+        self.rays[:, 17] *= numpy.abs(Rp)
+        self.rays[:, 14] += numpy.angle(Rp)
+
+    def apply_complex_reflectivities(self, Rs, Rp):
+        """
+
+        Parameters
+        ----------
+        Rs
+        Rp
+
+        Returns
+        -------
+
+        """
+        self.apply_complex_reflectivity_s(Rs)
+        self.apply_complex_reflectivity_p(Rp)
+
+    # phases
+    def add_phase_s(self, phase):
+        self.rays[:, 13] += phase
+
+    def add_phase_p(self, phase):
+        self.rays[:, 14] += phase
+
+    def add_phases(self, phase_s, phase_p):
+        self.add_phase_s(phase_s)
+        self.add_phase_p(phase_p)
     #
     #  interfaces like in shadow3
     #
