@@ -1,5 +1,5 @@
-from shadow4.syned.element_coordinates import ElementCoordinates
-from syned.beamline.beamline_element import BeamlineElement
+from shadow4.syned.beamline_element import BeamlineElement # todo: change when syned upgraded
+# from syned.beamline.beamline_element import BeamlineElement
 
 
 class S4BeamlineElement(BeamlineElement):
